require "open3"
require "james_bond/mission_build/build_handler/unable_to_find_dockerfile_error"

module JamesBond
  class MissionBuild
    class BuildHandler
      attr_accessor :config

      def initialize(command, config)
        @command       = command
        @config        = config
        @docker_config = config.docker_config
      end

      def run
        image_name      = build_image_name
        dockerfile_name = build_dockerfile_name
        raise(UnableToFindDockerfileError, dockerfile_path(dockerfile_name)) \
          if !dockerfile_exists?(dockerfile_name)

        command = "docker build -t #{image_name} -f #{dockerfile_name} ."
        puts "Building docker image \"#{image_name}\" with #{dockerfile_name}.. "
        Open3.popen3(command) do |stdin, stdout, stderr, thread|
          Thread.new do
            until (line = stdout.gets).nil? do
              puts line
            end
          end

          thread.join
        end
        puts "Done!"
      end

      private

      def build_image_name
        namespace = @docker_config["image"]["namespace"]
        name      = @docker_config["image"]["name"]
        tag       = build_tag
        "#{namespace}/#{name}:#{tag}"
      end

      def build_dockerfile_name
        @docker_config["dockerfile"]["pattern"].gsub("$environment", @command.env)
      end

      def build_tag
        @command.options[:tag] || "devel-#{Time.now.to_i}"
      end

      def dockerfile_exists?(dockerfile_name)
        File.file?(dockerfile_path(dockerfile_name))
      end

      def dockerfile_path(dockerfile_name)
        File.join(Dir.pwd, dockerfile_name)
      end
    end
  end
end
