require "yaml"
require "james_bond/mission_build/config/invalid_file_error"
require "james_bond/mission_build/config/no_config_available_error"

module JamesBond
  class MissionBuild
    class Config
      attr_accessor :yaml_path,
                    :raw_hash,
                    :docker_config

      def initialize(yaml_path:)
        @yaml_path = yaml_path
        extract_config_from_yaml(@yaml_path)

        validate_config
        @docker_config = @raw_hash["build"]["docker"]
      end

      private

      def extract_config_from_yaml(yaml_path)
        raise(self.class::InvalidFileError, yaml_path) if !file_exists?(yaml_path)
        @raw_hash = YAML.load(File.read(yaml_path)).freeze
      end

      def file_exists?(path)
        File.file?(path)
      end

      def validate_config
        build_config = (@raw_hash || {})["build"]
        raise(self.class::NoConfigAvailableError,
              { file_path: yaml_path, config_path: "build" }) if !build_config

        docker_config = build_config["docker"]
        raise(self.class::NoConfigAvailableError,
              { file_path: yaml_path, config_path: "build.docker" }) if !docker_config

        docker_image_config = docker_config["image"]
        raise(self.class::NoConfigAvailableError,
              { file_path: yaml_path, config_path: "build.docker.image" }) if !docker_image_config

        if !docker_image_config["namespace"]
          raise(self.class::NoConfigAvailableError,
                { file_path: yaml_path, config_path: "build.docker.image.namespace" })
        end
        if !docker_image_config["name"]
          raise(self.class::NoConfigAvailableError,
                { file_path: yaml_path, config_path: "build.docker.image.name" })
        end

        dockerfile_config = docker_config["dockerfile"]
        raise(self.class::NoConfigAvailableError,
              { file_path: yaml_path, config_path: "build.docker.dockerfile" }) if !dockerfile_config

        if !dockerfile_config["pattern"]
          raise(self.class::NoConfigAvailableError,
                { file_path: yaml_path, config_path: "build.docker.dockerfile.pattern" })
        end
      end
    end
  end
end
